from .backend import ImGuiBackend

from .utilities.PlaceManager import PlaceManager
from .utilities.PlacePanel import PlacePanel
from .utilities.ExplorerManager import ExplorerManager
from .utilities.SceneGraphExplorer import SceneGraphExplorer
from .utilities.TimeSliderManager import TimeSliderManager
from .utilities.IntervalTimeSlider import IntervalTimeSlider

__all__ = ['init',
           'ImGuiBackend',
           "PlacePanel",
           'PlaceManager',
           'ExplorerManager',
           'SceneGraphExplorer',
           'TimeSliderManager',
           'IntervalTimeSlider',
          ]

def init(window = None, parent = None,
         style = 'dark', wantPlaceManager = True,
         wantExplorerManager = True, wantTimeSliderManager = True):
    try:
        base.imgui
    except AttributeError:
        base.imgui = ImGuiBackend(window, parent, style)

        if wantPlaceManager:
            base.placeManager = PlaceManager()
        if wantExplorerManager:
            base.explorerManager = ExplorerManager()
        if wantTimeSliderManager:
            base.timeSliderManager = TimeSliderManager()



from direct.showbase.DirectObject import DirectObject
from panda3d.core import NodePath

from p3dimgui.utilities.SceneGraphExplorer import SceneGraphExplorer

from direct.extensions_native.extension_native_helpers import Dtool_funcToMethod

class ExplorerManager(DirectObject):
    def __init__(self):
        DirectObject.__init__(self)

        self.nodesToExplorers: dict[NodePath, SceneGraphExplorer] = {}

        # Replace the explore() extension method for NodePath
        def explore(nodePath):
            self.nodesToExplorers[nodePath] = SceneGraphExplorer(nodePath)

        Dtool_funcToMethod(explore, NodePath)
        self.accept('imgui-new-frame', self.__checkActive)

    def __checkActive(self):
        for node in list(self.nodesToExplorers.keys()):
            placer = self.nodesToExplorers[node]
            if not placer.active:
                del self.nodesToExplorers[node]




from direct.showbase.DirectObject import DirectObject
from panda3d.direct import CInterval

from p3dimgui.utilities.IntervalTimeSlider import IntervalTimeSlider

from direct.extensions_native.extension_native_helpers import Dtool_funcToMethod

class TimeSliderManager(DirectObject):
    def __init__(self):
        DirectObject.__init__(self)

        self.intervalToTimeSliders: dict[CInterval, IntervalTimeSlider] = {}

        # Replace the popupControls() extension method for CInterval
        def popupControls(interval):
            self.intervalToTimeSliders[interval] = IntervalTimeSlider(interval)

        Dtool_funcToMethod(popupControls, CInterval)
        Dtool_funcToMethod(popupControls, CInterval, "slider")
        self.accept('imgui-new-frame', self.__checkActive)

    def __checkActive(self):
        for node in list(self.intervalToTimeSliders.keys()):
            placer = self.intervalToTimeSliders[node]
            if not placer.active:
                del self.intervalToTimeSliders[node]



from imgui_bundle import imgui, imgui_ctx

from direct.showbase.DirectObject import DirectObject

class IntervalTimeSlider(DirectObject):
    def __init__(self, interval, active=True):
        DirectObject.__init__(self)
        self.interval = interval

        self.active = active

        self.__firstDraw = True
        self.accept('imgui-new-frame', self.__draw)

    def __draw(self):
        if not self.active:
            return
        curDuration = self.interval.getT()
        maxDuration = self.interval.getDuration()

        if self.__firstDraw:
            # imgui.set_next_window_content_size((784.0, 77.0))
            imgui.set_next_window_size((745,76))
            self.__firstDraw = False
        with imgui_ctx.begin(f"Time Slider \"{self.interval.getName()}\"", True,
                             imgui.WindowFlags_.no_resize.value | imgui.WindowFlags_.no_scrollbar.value | imgui.WindowFlags_.no_scroll_with_mouse.value) as (_, windowOpen):
            imgui.push_item_width(732)
            changed, value = imgui.slider_float(
                "##slider", curDuration,
                v_min=0.0, v_max=self.interval.getDuration(),
                format=f"{self.__calculateTimeFormat(curDuration)}/{self.__calculateTimeFormat(maxDuration)}",
                flags=imgui.SliderFlags_.no_input.value
            )
            imgui.pop_item_width()

            if not windowOpen:
                self.active = False
            if changed:
                self.interval.setT(value)

            imgui.spacing()
            imgui.same_line(spacing=312)
            if imgui.button("<<"):
                self.interval.setT(0)
                self.interval.pause()

            imgui.same_line()

            if imgui.button("Pause" if self.interval.isPlaying() else "Play"):
                if self.interval.isPlaying():
                    self.interval.pause()
                else:
                    self.interval.resume()

            imgui.same_line()

            if imgui.button(">>"):
                self.interval.setT(self.interval.getDuration())
                self.interval.pause()


    def __calculateTimeFormat(self, duration):
        if duration >= 60:
            self.currentTime = "00:00"
            totalSeconds = int(duration) % 60
            totalMinutes = int((duration - totalSeconds) / 60)

            if totalMinutes < 10:
                totalMinutes = "%s%s" % (0, totalMinutes)
            if totalSeconds < 10:
                totalSeconds = "%s%s" % (0, totalSeconds)

            return "%s:%s" % (totalMinutes, totalSeconds)
        else:
            if duration < 10:
                return "00:0%s" % (int(duration))
            else:
                return "00:%s" % (int(duration))
